import { addOrRemoveClass, append, prepend as prependFn, remove, createElement } from './utils/dom';
import { noop } from './utils/utils';
/**
 * Virtual slide component.
 */
export class Slide {
    constructor(html, frame, settings) {
        this.frame = frame;
        this.settings = settings;
        this.isMounted = false;
        this.isActive = false;
        this.hasChanged = false;
        if (typeof html === 'string') {
            this.html = html;
            return;
        }
        this.ref = html;
        this.html = this.ref.innerHTML;
        this.isMounted = true;
    }
    set(property, value) {
        this[property] = value;
        this.hasChanged = true;
    }
    render() {
        const element = createElement('div', { classNames: 'virchual__slide', html: this.html });
        this.setAttributes(element);
        return element;
    }
    mount(prepend = false) {
        if (this.isMounted) {
            // slide has changed -> update in DOM
            if (this.hasChanged) {
                this.update();
            }
            return;
        }
        console.debug('[Mount] Slide', { ref: this.ref, prepend });
        this.ref = this.render();
        this.isMounted = true;
        this.ref.addEventListener('transitionend', this.transitionEndCallback);
        const insert = prepend ? prependFn : append;
        insert(this.frame, this.ref);
    }
    unmount() {
        console.debug('[Unmount] Slide', { ref: this.ref });
        this.isMounted = false;
        remove(this.ref);
    }
    /**
     * Start transition.
     *
     * @param value
     * @param done
     */
    translate(value, done = noop) {
        this.transitionEndCallback = done;
        this.ref.style.transition = `transform ${this.settings.speed}ms ${this.settings.easing}`;
        this.ref.style.transform = `translate3d(calc(${this.position}% + ${Math.round(value)}px), 0, 0)`;
    }
    update() {
        this.hasChanged = false;
        this.setAttributes(this.ref);
    }
    setAttributes(element) {
        addOrRemoveClass(element, 'virchual__slide--active', !this.isActive);
        element.style.transform = `translate3d(${this.position}%, 0, 0)`;
    }
}
//# sourceMappingURL=slide.js.map