import './css/styles.css';
import { identity } from './types';
export declare type VirchualSettings = {
    slides?: string[] | (() => string[]);
    speed?: number;
    easing?: string;
    swipeDistanceThreshold?: number;
    flickVelocityThreshold?: number;
    flickPower?: number;
    pagination?: boolean;
    window?: number;
};
export declare class Virchual {
    settings: VirchualSettings;
    container: HTMLElement;
    frame: HTMLElement;
    paginationButtons: HTMLButtonElement[];
    currentIndex: number;
    private slides;
    private eventBus;
    private isBusy;
    private pagination;
    private eventBindings;
    constructor(element: HTMLElement | string, settings?: VirchualSettings);
    /**
     * Mount components.
     */
    mount(): void;
    /**
     * Register callback fired on the given event(s).
     *
     * @param events An event name. Use space to separate multiple events.
     *                             Also, namespace is accepted by dot, such as 'resize.{namespace}'.
     * @param handler A callback function.
     * @param elm  Optional. Native event will be listened to when this arg is provided.
     * @param opts Optional. Options for addEventListener.
     */
    on(events: string, handler: identity, elm?: (Window & typeof globalThis) | Element, opts?: Record<string, unknown>): void;
    /**
     * Unsubscribe the given event.
     *
     * @param events - A event name.
     * @param elm    - Optional. removeEventListener() will be called when this arg is provided.
     */
    off(events: string, elm?: (Window & typeof globalThis) | Element): void;
    /**
     * Go to previous slide.
     */
    prev(): void;
    /**
     * Go to next slide.
     */
    next(): void;
    private goTo;
    private hydrate;
    /**
     * Mount and unmount slides.
     */
    private mountAndUnmountSlides;
    private bindEvents;
    /**
     * Called when frame is clicked.
     *
     * @param event A click event.
     */
    private onClick;
    private onPaginationButtonClick;
    /**
     * Handle drag event.
     *
     * @param event
     */
    private onDrag;
    /**
     * Handle dragged event.
     *
     * @param event
     */
    private onDragEnd;
}
